################################################################################
# Kube module, A content wrapper object for use with Zope Page Templates.
#
# This is Open Source Software.
# Copyright 2002 Isaac Levy, under the terms of the BSD open liscense.
# Office of Structured Systems, in joint cooperation with the
# Informatikbro [tactical division]
#
# This product includes software developed by Digital Creations/ ZOPE, INC
# for use in the Z Object Publishing Environment
#      (http://www.zope.org/).
#      Much of the code was initally basd on the LockableFolder Product.
# Please continue to support Zope, Python, 
# and other projects that dare to think differently.
# 
#     ##########################################################################
#     # Disclaimer
#     #
#     # THIS SOFTWARE IS PROVIDED BY ISAAC LEVY ``AS IS'' AND ANY
#     # EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#     # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#     # PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL ISAAC LEVY OR HIS
#     # CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#     # SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#     # LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#     # USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#     # ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#     # OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#     # OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#     # SUCH DAMAGE.
# 
# 
from OFS import Folder, PropertyManager, SimpleItem, FindSupport
from AccessControl import ClassSecurityInfo, getSecurityManager, Role
from Globals import InitializeClass, DTMLFile
from Acquisition import Implicit
from Globals import Persistent
from Globals import MessageDialog
from AccessControl.Role import RoleManager
from OFS.SimpleItem import Item
from OFS.PropertyManager import PropertyManager
from OFS.FindSupport import FindSupport
from Persistence import PersistentMapping

#############################
# Constructor form and method
#
#
addKubeForm=DTMLFile('dtml/KubeAdd', globals())
editKubeForm=DTMLFile('dtml/KubeProperties', globals())
# much like a Zope Folder,
def manage_addKube(self, id, title='', 
                     author='', 
                     content_creation_date='', 
                     keywords='', 
                     description='', 
                     full_content='', 
                     copyright='', 
                     comment1='', 
                     comment2='', 
                     live=0, 
                     createUserF=0, 
                     REQUEST=None):
    """
    If no ID is provided, return id as the time as a floating 
    point number expressed in seconds since the epoch, in UTC.
    (On UNIX, epoch is 1970.)
    """
    ob=Kube()
# simple time-based unique id generator:
    if not id:
        """What time is it?"""
        idstart = time.time()
        fmt = "%f"
        context = self
        while hasattr(context, fmt % idstart):
            idstart = idstart + .00001
        id = fmt % idstart
    ob.id=str(id) 
    ob.title=title
    ob.author=str(author)
    ob.content_creation_date=str(content_creation_date)
    ob.keywords=str(keywords)
    ob.description=str(description)
    ob.full_content=str(full_content)
    ob.copyright=str(copyright)
    ob.comment1=str(comment1)
    ob.comment2=str(comment2)
    ob.live=(live)
    self._setObject(id, ob)
    ob=self._getOb(id)

    checkPermission=getSecurityManager().checkPermission    

    if createUserF:
        if not checkPermission('Add User Folders', ob):
            raise 'Unauthorized', (
                  'You are not authorized to add User Folders.'
                  )
        ob.manage_addUserFolder()

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)

################################################################################
# Product class
#
#
class Kube(PropertyManager, Folder.Folder):
    """
    Kubes are basic container objects that provide a property-based
    interface for object management. Kube objects are much like Folder 
    objects, and implement a html management interface.
    """
    meta_type='Kube'
    icon = 'images/kubeicon.gif'
    _properties = (
        {'id':'title', 'type':'string', 'mode':'w'},
        {'id':'author', 'type':'string', 'mode':'wd'},
        {'id':'content_creation_date', 'type':'string', 'mode':'wd'},
        {'id':'keywords', 'type':'string', 'mode':'wd'},
        {'id':'description', 'type':'text', 'mode':'wd'},
        {'id':'full_content', 'type':'text', 'mode':'wd'},
        {'id':'copyright', 'type':'string', 'mode':'wd'},
        {'id':'comment1', 'type':'string', 'mode':'wd'},
        {'id':'comment2', 'type':'string', 'mode':'wd'},
        {'id': 'live', 'type':'boolean', 'mode':'wd'},
    )


    manage_main=DTMLFile('dtml/main', globals(),
                          property_extensible_schema__=1)
    manage_attach=DTMLFile('dtml/attach', globals())
    manage_kube=DTMLFile('dtml/main', globals())
    manage_options=(
        (
        {'label':'Kube', 'action':'manage_kube',},
        {'label':'Attachments', 'action':'manage_attach',
         'help':('OFSP','Folder_View.stx')},
        {'label':'View', 'action':'index_html',
         'help':('OFSP','Folder_View.stx')},
        )
        +PropertyManager.manage_options
        +RoleManager.manage_options
        +Item.manage_options
        +FindSupport.manage_options
        )
    security=ClassSecurityInfo()

    def __init__(self):
        self._access = PersistentMapping()

    def _setOb(self, id, object):
        Folder.Folder._setOb(self, id, object)
        self._access[id] = 0
        
    def _delOb(self, id):
        Folder.Folder._delOb(self, id)
        del self._access[id]
        
    def _lockAccess(self, id):
        self._access[id] = 1
    def _unlockAccess(self, id):
        self._access[id] = 0
    def _isLocked(self, id):
        return self._access.get(id,None)



    __ac_permissions__=(
        ('View management screens',
         ('manage', 'manage_main',)),
        ('Change Kubes',
         ('manage_edit','manage_upload','PUT')),
        ('Manage Kube properties', ('manage_addProperty',
                                    'manage_editProperties',
                                    'manage_delProperties',
                                    'manage_changeProperties'
                                    'manage_kube')),
        ('View',
         ('index_html', 'view_image_or_file', 'get_size',
          'getContentType', '')),
        ('FTP access',
         ('manage_FTPstat','manage_FTPget','manage_FTPlist')),
        ('Delete objects',
         ('DELETE',)),
        )

##########################
# Initialize Product class
#
# 
InitializeClass(Kube)
